<?php

namespace Research\Slide\Policies;

use Litepie\User\Contracts\UserPolicy;
use Research\Slide\Models\Slide;

class SlidePolicy
{

    /**
     * Determine if the given user can view the slide.
     *
     * @param UserPolicy $user
     * @param Slide $slide
     *
     * @return bool
     */
    public function view(UserPolicy $user, Slide $slide)
    {
        if ($user->canDo('slide.slide.view') && $user->isAdmin()) {
            return true;
        }

        return $slide->user_id == user_id() && $slide->user_type == user_type();
    }

    /**
     * Determine if the given user can create a slide.
     *
     * @param UserPolicy $user
     * @param Slide $slide
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('slide.slide.create');
    }

    /**
     * Determine if the given user can update the given slide.
     *
     * @param UserPolicy $user
     * @param Slide $slide
     *
     * @return bool
     */
    public function update(UserPolicy $user, Slide $slide)
    {
        if ($user->canDo('slide.slide.edit') && $user->isAdmin()) {
            return true;
        }

        return $slide->user_id == user_id() && $slide->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given slide.
     *
     * @param UserPolicy $user
     * @param Slide $slide
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Slide $slide)
    {
        return $slide->user_id == user_id() && $slide->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given slide.
     *
     * @param UserPolicy $user
     * @param Slide $slide
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Slide $slide)
    {
        if ($user->canDo('slide.slide.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given slide.
     *
     * @param UserPolicy $user
     * @param Slide $slide
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Slide $slide)
    {
        if ($user->canDo('slide.slide.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
